package com.supervision.handler.graph;

import cn.hutool.core.collection.CollUtil;
import com.supervision.exception.ItemNotFoundException;
import com.supervision.ngbatis.dao.CommonQueryDao;
import com.supervision.ngbatis.domain.tag.ItemBranch;
import com.supervision.ngbatis.domain.tag.ItemLeaf;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 查询图数据库,寻找符合条件的节点
 */
@Slf4j
@Component
public class FindItemNodeHandler {

    @Resource
    private CommonQueryDao commonQueryDao;

    /**
     * 找到抽取的事项所匹配的叶子节点列表
     *
     * @param itemNameList 匹配到的事项
     * @return 匹配的叶子节点列表
     */
    public List<ItemLeaf> findAllMatchLeafNode(List<String> itemNameList) {
        log.info("根据事项名称{}寻找可能匹配的节点", CollUtil.join(itemNameList, ";"));
        List<ItemLeaf> matchLeafNodeList = new ArrayList<>();
        for (String itemName : itemNameList) {
            // 首先查询叶子节点的数据,如果有,就直接按照叶子节点的数据来匹配
            List<ItemLeaf> leafList = commonQueryDao.findItemLeafNodeVertexByItemName(itemName);
            if (CollUtil.isEmpty(leafList)) {
                List<ItemBranch> itemBranchNodeVertexByItemName = commonQueryDao.findItemBranchNodeVertexByItemName(itemName);
                // 如果分支节点不为空,那么就从分支节点去找叶子节点
                if (CollUtil.isNotEmpty(itemBranchNodeVertexByItemName)) {
                    List<ItemLeaf> leafListTemp = new ArrayList<>();
                    // 遍历分支节点,找到叶子节点
                    for (ItemBranch itemBranch : itemBranchNodeVertexByItemName) {
                        List<ItemLeaf> itemLeafByBranchId = commonQueryDao.findItemLeafByBranchId(itemBranch.getVid());
                        if (CollUtil.isNotEmpty(itemLeafByBranchId)) {
                            leafListTemp.addAll(itemLeafByBranchId);
                        }
                    }
                    leafList.addAll(leafListTemp);
                }
            }
            matchLeafNodeList.addAll(leafList);
        }

        if (CollUtil.isEmpty(matchLeafNodeList)) {
            throw new ItemNotFoundException("未找到与事项匹配的节点");
        }
        return matchLeafNodeList;
    }

    /**
     * 直接把所有的节点全部找到,带节点的形式去匹配
     *
     * @return
     */
    public List<String> findAllItemNode() {
        log.info("寻找可能匹配的节点");
        // 首先查询叶子节点的数据,如果有,就直接按照叶子节点的数据来匹配
        List<ItemLeaf> leafList = commonQueryDao.findAllItemLeafNodeVertex();
        List<ItemBranch> allItemBranchNodeVertex = commonQueryDao.findAllItemBranchNodeVertex();
        List<String> itemNameList = leafList.stream().map(ItemLeaf::getItemName).distinct().collect(Collectors.toList());
        Set<String> branchNodeName = allItemBranchNodeVertex.stream().map(ItemBranch::getItemName).collect(Collectors.toSet());
        itemNameList.addAll(branchNodeName);
        return itemNameList;
    }
}
